// -------------------------------------------------------
// This file is part of the OpenNETCF.Telephony Library
// 2007-2009 OpenNETCF Consulting, LLC
// www.opennetcf.com
//
// For license information visit http://tapi.codeplex.com
// -------------------------------------------------------

using System.Runtime.InteropServices;

namespace OpenNETCF.Telephony
{
    /// <summary>
    /// This structure describes parameters supplied when making calls using the lineMakeCall function. 
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    public class CallParameters : PhoneBase            // Defaults:
    {
        /// <summary>
        /// 
        /// </summary>
        public CallParameters(int size)
            : base(size)
        {
            TotalSize = Data.Length;
        }

        /// <summary>
        /// 
        /// </summary>
        protected override int MinimumSize { get { return 1024; } }

        /// <summary>
        /// 
        /// </summary>
        public string CalledParty { get { return GetString(CalledPartyOffset, CalledPartySize); } }
        /// <summary>
        /// 
        /// </summary>
        public string Comment { get { return GetString(CommentOffset, CommentSize); } }
        /// <summary>
        /// 
        /// </summary>
        public string TargetAddress { get { return GetString(TargetAddressOffset, TargetAddressSize); } }

        /// <summary>
        /// Total size, in bytes, allocated to this data structure. 
        /// </summary>
        public int TotalSize;                    // ---------
        /// <summary>
        /// Value that specifies the bearer mode for the call. 
        /// </summary>
        public BearerMode BearerMode;                   // voice
        /// <summary>
        /// Value that specifies the data rate range requested for the call's data stream in bps (bits per second). 
        /// </summary>
        public int MinimumRate;                      // (3.1kHz)
        /// <summary>
        /// Value that specifies the data rate range requested for the call's data stream in bps (bits per second). 
        /// </summary>
        public int MaximumRate;                      // (3.1kHz)
        /// <summary>
        /// Value that specifies the expected media mode of the call. 
        /// </summary>
        public MediaMode MediaMode;                    // interactiveVoice
        /// <summary>
        /// Value that specifies a collection of Boolean call-setup parameters. 
        /// </summary>
        public CallParameterFlags CallParamFlags;               // 0
        /// <summary>
        /// Value that specifies the mode by which the originating address is specified. 
        /// </summary>
        internal AddressMode AddressMode;                  // addressID
        /// <summary>
        /// Address identifier of the originating address if AddressMode is set to AddressId.
        /// </summary>
        public int AddressID;                    // (any available)
        /// <summary>
        /// Unsupported; set to zero.
        /// </summary>
        public DialParameters DialParams;                 // (0, 0, 0, 0)
        /// <summary>
        /// Size, in bytes, of the field holding the originating address.
        /// </summary>
        public int OriginatingAddressSize;              // 0
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure. 
        /// </summary>
        public int OriginatingAddressOffset;
        /// <summary>
        /// Specifies the displayable string that is used for logging purposes. 
        /// </summary>
        public int DisplayableAddressSize;
        /// <summary>
        /// Specifies the displayable string that is used for logging purposes. 
        /// </summary>
        public int DisplayableAddressOffset;
        /// <summary>
        /// Size, in bytes, of the field that holds called-party data.
        /// </summary>
        public int CalledPartySize;              // 0
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure. 
        /// </summary>
        public int CalledPartyOffset;
        /// <summary>
        /// Size, in bytes, of the field that holds comments about the call.
        /// </summary>
        public int CommentSize;                  // 0
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int CommentOffset;
        /// <summary>
        /// Size, in bytes, of the field that holds user-user data.
        /// </summary>
        public int UserUserInfoSize;             // 0
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int UserUserInfoOffset;
        /// <summary>
        /// Size, in bytes, of the field that holds high-level compatibility data.
        /// </summary>
        public int HighLevelCompSize;            // 0
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int HighLevelCompOffset;
        /// <summary>
        /// Size, in bytes, of the field that holds low-level compatibility data.
        /// </summary>
        public int LowLevelCompSize;             // 0
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int LowLevelCompOffset;
        /// <summary>
        /// Size, in bytes, of the field that holds device-specific data.
        /// </summary>
        public int DeviceSpecificSize;              // 0
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int DeviceSpecificOffset;
        /// <summary>
        /// 
        /// </summary>
        public int PredictiveAutoTransferStates;                 // TAPI v2.0
        /// <summary>
        /// The CallState values, entry into which cause the call to be blind-transferred to the specified target address. Set to zero if automatic transfer is not desired.
        /// </summary>
        public int TargetAddressSize;                            // TAPI v2.0
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int TargetAddressOffset;                          // TAPI v2.0
        /// <summary>
        /// Total size, in bytes, of a Winsock2 FLOWSPEC structure followed by Winsock2 provider-specific data.
        /// </summary>
        public int SendingFlowspecSize;                          // TAPI v2.0
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int SendingFlowspecOffset;                        // TAPI v2.0
        /// <summary>
        /// Total size, in bytes, of a Winsock2 FLOWSPEC structure.
        /// </summary>
        public int ReceivingFlowspecSize;                        // TAPI v2.0
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int ReceivingFlowspecOffset;                      // TAPI v2.0
        /// <summary>
        /// Size, in bytes, of a null-terminated ASCII string (the size includes the NULL) that indicates the device class of the device whose configuration is specified in DeviceConfig.
        /// </summary>
        public int DeviceClassSize;                              // TAPI v2.0
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int DeviceClassOffset;                            // TAPI v2.0
        /// <summary>
        /// Size, in bytes, of the opaque configuration data structure pointed to by DeviceConfigOffset. 
        /// </summary>
        public int DeviceConfigSize;                             // TAPI v2.0
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int DeviceConfigOffset;                           // TAPI v2.0
        /// <summary>
        /// Size, in bytes, of the application-settable call data to be initially attached to the call.
        /// </summary>
        public int CallDataSize;                                 // TAPI v2.0
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int CallDataOffset;                               // TAPI v2.0
        /// <summary>
        /// Number of seconds, after the completion of dialing, that the call should wait in the PROCEEDING or RINGBACK state, before abandoned by the service provider
        /// </summary>
        public int NoAnswerTimeout;                              // TAPI v2.0
        /// <summary>
        /// Size, in bytes, of a null-terminated ASCII string (the size includes the NULL) that specifies the identity of the party placing the call.
        /// </summary>
        public int CallingPartyIDSize;                           // TAPI v2.0
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure.
        /// </summary>
        public int CallingPartyIDOffset;                         // TAPI v2.0
        /// <summary>
        /// Address type used for the call. This member is available only if the negotiated TAPI version is 3.0 or later.
        /// </summary>
        public int AddressType;                                  // TAPI v3.0
    }
}
