// -------------------------------------------------------
// This file is part of the OpenNETCF.Telephony Library
// 2007-2009 OpenNETCF Consulting, LLC
// www.opennetcf.com
//
// For license information visit http://tapi.codeplex.com
// -------------------------------------------------------

using System;
using System.Runtime.InteropServices;
using System.Text;

namespace OpenNETCF.Telephony
{
    /// <summary>
    /// This is a service-provider-specific configuration structure. It is specific to version 1 of this Telephony Service Provider (TSP). Future versions may append additional data elements to the end of this structure.
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    public class DeviceConfiguration : PhoneBase
    {
        /// <summary>
        /// Creates a DeviceConfiguration instance
        /// </summary>
        /// <param name="size"></param>
        public DeviceConfiguration(int size)
            : base(size)
        {
            TotalSize = size;
        }
        /// <summary>
        /// The total size, in bytes, allocated to this data structure. 
        /// </summary>
        public int TotalSize;               /* Standard TAPI structure variable */
        /// <summary>
        /// The total size, in bytes, allocated to this data structure. 
        /// </summary>
        public int NeededSize;              /* Standard TAPI structure variable */
        /// <summary>
        /// The total size, in bytes, allocated to this data structure. 
        /// </summary>
        public int UsedSize;                /* Standard TAPI structure variable */
        /// <summary>
        /// TRUE if and only if BearerInfo is valid. 
        /// </summary>
        public int BearerInfoValid;          /* TRUE iff sbiBearerInfo is valid */
        /// <summary>
        /// Bearer structure.
        /// </summary>
        public BearerInformation BearerInfo;             /* Bearer structure */
        /// <summary>
        /// TRUE if and only if cdciDataCompInfo is valid. 
        /// </summary>
        public int DataCompInfoValid;        /* TRUE iff sdciDataCompInfo is valid */
        /// <summary>
        /// Data compression structure.
        /// </summary>
        public DataCompressionInfo DataCompInfo;          /* Data compression structure */
        /// <summary>
        /// TRUE if and only if crliRadioLinkInfo is valid.
        /// </summary>
        public int RadioLinkInfoValid;       /* TRUE iff srliRadioLinkInfo is valid */
        /// <summary>
        /// Radio Link Protocol (RLP) structure.
        /// </summary>
        public RadioLinkProtocolSettings RadioLinkInfo;         /* RLP structure */
        /// <summary>
        /// TRUE if and only if cgciGPRSConnectionInfo is valid. 
        /// </summary>
        public int GPRSConnectionInfoValid;  /* TRUE iff sgciGPRSConnectionInfo is valid */
        /// <summary>
        /// General Packet Radio Service (GPRS) connection structure. 
        /// </summary>
        public GPRSConnectionInfo GPRSConnectionInfo;    /* GPRS connection structure */
        /// <summary>
        /// Not supported for Smartphone.
        /// </summary>
        public int Flags;                   /* Bit-mask of CELLDEVCONFIG_FLAG_* constants */
    }
}
