// -------------------------------------------------------
// This file is part of the OpenNETCF.Telephony Library
// 2007-2009 OpenNETCF Consulting, LLC
// www.opennetcf.com
//
// For license information visit http://tapi.codeplex.com
// -------------------------------------------------------

using System;
using System.Runtime.InteropServices;
using System.Text;

namespace OpenNETCF.Telephony
{
    /// <summary>
    /// This structure describes the current status of a line device
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    internal class DeviceStatus : PhoneBase
    {
        public DeviceStatus(int nSize)
            : base(nSize)
        {
            TotalSize = nSize;
        }

        protected override int MinimumSize { get { return 1024; } }

        /// <summary>
        /// Total size in bytes allocated to this data structure. 
        /// </summary>
        public int TotalSize;
        /// <summary>
        /// Size, in bytes, for this data structure that is needed to hold all the returned data.
        /// </summary>
        public int NeededSize;
        /// <summary>
        /// Size, in bytes, of the portion of this data structure that contains useful data. 
        /// </summary>
        public int UsedSize;
        /// <summary>
        /// Number of active opens on the line device.
        /// </summary>
        public int OpenCount;
        /// <summary>
        /// Value that specifies a bit array that indicates for which media modes the line device is currently open.
        /// </summary>
        public int OpenMediaModes;
        /// <summary>
        /// Number of calls on the line in call states other than idle, onhold, onholdpendingtransfer, and onholdpendingconference. 
        /// </summary>
        public int ActiveCallCount;
        /// <summary>
        /// Number of calls on the line in the onhold state. 
        /// </summary>
        public int OnHoldCallCount;
        /// <summary>
        /// Number of calls on the line in the onholdpendingtransfer or onholdpendingconference state.
        /// </summary>
        public int OnHoldPendCallCount;
        /// <summary>
        /// Specifies the line-related API functions that are currently available on this line. 
        /// </summary>
        public LineFeature LineFeatures;
        /// <summary>
        /// Number of outstanding call completion requests on the line. 
        /// </summary>
        public int CallCompletionCount;
        /// <summary>
        /// Current ring mode on the line device. 
        /// </summary>
        public int RingMode;
        /// <summary>
        /// Current signal level of the connection on the line. This is a value in the range 0x00000000 (weakest signal) to 0x0000FFFF (strongest signal). 
        /// </summary>
        public int SignalLevel;
        /// <summary>
        /// Current battery level of the line device hardware. This is a value in the range 0x00000000 (battery empty) to 0x0000FFFF (battery full). 
        /// </summary>
        public int BatteryLevel;
        /// <summary>
        /// Specifies the current roam mode of the line device.
        /// </summary>
        public RoamMode RoamMode;
        /// <summary>
        /// Size, in bytes, of this data structure that contains useful data.
        /// </summary>
        public StatusFlags Flags;
        /// <summary>
        /// Size, in bytes, of the variably sized device field that contains an array with DWORD-sized entries. 
        /// </summary>
        public int TerminalModesSize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure. 
        /// </summary>
        public int TerminalModesOffset;
        /// <summary>
        /// Size, in bytes, of the variably sized device-specific field. 
        /// </summary>
        public int DevSpecificSize;
        /// <summary>
        /// Offset, in bytes, from the beginning of this data structure. 
        /// </summary>
        public int DevSpecificOffset;
    }
}
