// -------------------------------------------------------
// This file is part of the OpenNETCF.Telephony Library
// 2007-2009 OpenNETCF Consulting, LLC
// www.opennetcf.com
//
// For license information visit http://tapi.codeplex.com
// -------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace OpenNETCF
{
    class Environment2
    {
        //maximum supported length of a file path
        private const int MaxPath = 260;

        #region Get Folder Path
        /// <summary>
        /// Gets the path to the system special folder identified by the specified enumeration.
        /// </summary>
        /// <param name="folder">An enumerated constant that identifies a system special folder.</param>
        /// <returns>The path to the specified system special folder, if that folder physically exists on your computer; otherwise, the empty string ("").
        /// A folder will not physically exist if the operating system did not create it, the existing folder was deleted, or the folder is a virtual directory, such as My Computer, which does not correspond to a physical path.</returns>
        /// <seealso cref="M:System.Environment.GetFolderPath(System.Environment.SpecialFolder)">System.Environment.GetFolderPath Method</seealso>
        public static string GetFolderPath(SpecialFolder folder)
        {
            StringBuilder path = new StringBuilder(MaxPath + 2);

            SHGetSpecialFolderPath(IntPtr.Zero, path, (int)folder, 0);
     
            return path.ToString();
        }


        #endregion

        [DllImport("coredll.dll", EntryPoint = "SHGetSpecialFolderPath", SetLastError = false)]
        internal static extern bool SHGetSpecialFolderPath(IntPtr hwndOwner, System.Text.StringBuilder lpszPath, int nFolder, int fCreate);
    }

      


    #region Special Folder
    /// <summary>
    /// Specifies enumerated constants used to retrieve directory paths to system special folders.
    /// </summary>
    /// <remarks>Not all platforms support all of these constants.</remarks>
    /// <seealso cref="T:System.Environment.SpecialFolder">System.Environment.SpecialFolder Enumeration</seealso>
    public enum SpecialFolder
    {
        // <summary>
        // The logical Desktop rather than the physical file system location.
        // </summary>
        //Desktop		= 0x00,
        /// <summary>
        /// The directory that contains the user's program groups.
        /// </summary>
        Programs = 0x02,
        // <summary>
        // control panel icons
        // </summary>
        //Controls		= 0x03,
        // <summary>
        // printers folder
        // </summary>
        //Printers		= 0x04,
        /// <summary>
        /// The directory that serves as a common repository for documents.
        /// </summary>
        Personal = 0x05,
        /// <summary>
        /// The "My Documents" folder.
        /// </summary>
        MyDocuments = 0x5,
        /// <summary>
        /// The directory that serves as a common repository for the user's favorite items.
        /// </summary>
        Favorites = 0x06,
        /// <summary>
        /// The directory that corresponds to the user's Startup program group.
        /// The system starts these programs whenever a user starts Windows CE.
        /// </summary>
        Startup = 0x07,
        /// <summary>
        /// The directory that contains the user's most recently used documents.
        /// <para><b>Not supported in Windows Mobile.</b></para>
        /// </summary>
        Recent = 0x08,
        // <summary>
        // The directory that contains the Send To menu items.
        // </summary>
        //SendTo			= 0x09,
        // <summary>
        // Recycle bin.
        // </summary>
        //RecycleBin		= 0x0A,
        /// <summary>
        /// The directory that contains the Start menu items.
        /// </summary>
        StartMenu = 0x0B,
        /// <summary>
        /// The "My Music" folder.
        /// <para><b>Supported only on Windows Mobile.</b></para>
        /// </summary>
        MyMusic = 0xd,
        // <summary>
        // The "My Video" folder.
        // <para><b>Supported only on Windows Mobile.</b></para>
        // </summary>
        //MyVideo = 0xe,
        /// <summary>
        /// The directory used to physically store file objects on the desktop.
        /// Do not confuse this directory with the desktop folder itself, which is a virtual folder.
        /// <para><b>Not supported in Windows Mobile.</b></para>
        /// </summary>
        DesktopDirectory = 0x10,
        // <summary>
        // The "My Computer" folder.
        // </summary>
        //MyComputer		= 0x11,
        // <summary>
        // Network Neighbourhood
        // <para><b>Not supported in Windows Mobile.</b></para></summary>
        //NetworkNeighborhood = 0x12,
        /// <summary>
        /// The Fonts folder.
        /// </summary>
        Fonts = 0x14,
        /// <summary>
        /// The directory that serves as a common repository for application-specific data for the current user.
        /// </summary>
        ApplicationData = 0x1a,
        /// <summary>
        /// The Windows folder.
        /// </summary>
        Windows = 0x24,
        /// <summary>
        /// The program files directory.
        /// </summary>
        ProgramFiles = 0x26,
        /// <summary>
        /// The "My Pictures" folder.
        /// <para><b>Supported only on Windows Mobile.</b></para>
        /// </summary>
        MyPictures = 0x27,
    }
    #endregion
}
