// -------------------------------------------------------
// This file is part of the OpenNETCF.Telephony Library
// 2007-2009 OpenNETCF Consulting, LLC
// www.opennetcf.com
//
// For license information visit http://tapi.codeplex.com
// -------------------------------------------------------

using System;
using System.Runtime.InteropServices;
using System.Text;

namespace OpenNETCF.Telephony
{
    /// <summary>
    /// This structure describes parameters supplied when making calls using lineInitializeEx. 
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    public class InitializeParameters2
    {
        public InitializeParameters2(IntPtr hEvent)
        {
            this.EventOrPort = hEvent;
            TotalSize = Marshal.SizeOf(typeof(InitializeParameters2));
            Options = InitializeParameterOptions.UseEvent;
        }
        /// <summary>
        /// Total size, in bytes, allocated to this data structure.
        /// </summary>
        public int TotalSize;
        /// <summary>
        /// Size, in bytes, for this data structure that is needed to hold all the returned information.
        /// </summary>
        public int NeededSize;
        /// <summary>
        /// Size, in bytes, of the portion of this data structure that contains useful information. 
        /// </summary>
        public int UsedSize;
        /// <summary>
        /// Specifies the event notification mechanism the application desires to use. 
        /// </summary>
        public InitializeParameterOptions Options;
        //If Options specifies InitializeParameterOptions.UseEvent, TAPI returns the event handle in this field, else it returns the completion port handle.
        public IntPtr EventOrPort;
        /// <summary>
        /// Completion key.
        /// </summary>
        public int CompletionKey;
    }
}
