// -------------------------------------------------------
// This file is part of the OpenNETCF.Telephony Library
// 2007-2009 OpenNETCF Consulting, LLC
// www.opennetcf.com
//
// For license information visit http://tapi.codeplex.com
// -------------------------------------------------------

using System;
using System.Runtime.InteropServices;

namespace OpenNETCF.Telephony
{

    /// <summary>
	/// TAPI P/Invoke definitions
	/// </summary>
	public class NativeMethods
	{
        [DllImport("coredll", SetLastError=true)]
		extern static public int lineInitializeEx(
			out IntPtr lpm_hLineApp, 
			IntPtr hInstance, 
			IntPtr lpfnCallback, 
			string lpszFriendlyAppName,
			out int lpdwNumDevs,
			ref int lpdwAPIVersion,
			InitializeParameters2 lpLineInitializeExParams);

		[DllImport("coredll", SetLastError=true)]
		extern static public int lineOpen(IntPtr m_hLineApp, 
			int dwDeviceID, 
			out IntPtr lphLine, 
			int dwAPIVersion, 
			int dwExtVersion, 
			IntPtr dwCallbackInstance, 
			CallPrivilege dwPrivileges, 
			MediaMode dwMediaModes, 
			CallParameters lpCallParams);

		[DllImport("coredll", SetLastError=true)]
		extern static public int lineOpen(IntPtr m_hLineApp, 
			int dwDeviceID, 
			out IntPtr lphLine, 
			int dwAPIVersion, 
			int dwExtVersion, 
			IntPtr dwCallbackInstance, 
			CallPrivilege dwPrivileges, 
			MediaMode dwMediaModes, 
			IntPtr lpCallParams);

		[DllImport("coredll", SetLastError=true)]
		extern static public int lineClose(IntPtr hLine);

		[DllImport("coredll", SetLastError=true)]
		extern static public int lineGetCallInfo( IntPtr hCall, byte[] lpCallInfo );

		[DllImport("coredll", SetLastError=true)]
		extern static public int lineDeallocateCall( IntPtr hCall );

		[DllImport("coredll", SetLastError=true)]
		extern static public int lineGetMessage(IntPtr m_hLineApp, out LineMessage lpMessage, int dwTimeout );

		[DllImport("coredll", SetLastError=true)]
		extern static public int lineShutdown(IntPtr m_hLineApp);

		[DllImport("coredll", SetLastError=true)]
		extern static public int lineNegotiateAPIVersion(
			IntPtr m_hLineApp, 
			int dwDeviceID, 
			int dwAPILowVersion, 
			int dwAPIHighVersion, 
			out int lpdwAPIVersion, 
			out ExtensionID lpExtensionID
			);

		[DllImport("coredll", SetLastError=true)]
		extern static public int lineGetDevCaps(IntPtr m_hLineApp, 
			int dwDeviceID, 
			int dwAPIVersion, 
			int dwExtVersion, 
			byte[] lpLineDevCaps
			);

        /// <summary>
        /// This function sets the application's privilege to the specified privilege. 
        /// 
        /// New in v1.1
        /// </summary>
        /// <param name="hCall">Handle to the call</param>
        /// <param name="dwPrivilege">the CallPrivilege to set the call to</param>
        /// <returns></returns>
        /// <version>1.1</version>
        [DllImport("coredll", SetLastError = true)]
        extern static public int lineSetCallPrivilege(IntPtr hCall, CallPrivilege dwPrivilege);

		[DllImport("coredll", SetLastError=true)]
		extern static public int lineMonitorDigits(	IntPtr hCall, DigitMode dwDigitModes );

		[DllImport("coredll", SetLastError=true)]
		extern static public int lineGetDevConfig(
			int dwDeviceID,
			byte[] lpDeviceConfig,
			string lpszDeviceClass
			);

		[DllImport("coredll", SetLastError=true)]
		extern static public int lineMakeCall(
			IntPtr hLine, 
			out IntPtr lphCall, 
			string lpszDestAddress, 
			int dwCountryCode, 
			byte[] lpCallParams
			);

		[DllImport("coredll", SetLastError=true)]
		extern static public int lineHold(IntPtr hCall);

		[DllImport("coredll", SetLastError=true)]
		extern static public int lineGetCallStatus( IntPtr hCall, /*LPLINECALLSTATUS*/ byte[] lpCallStatus );

		[DllImport("coredll", SetLastError=true)]
		extern static public int lineDrop(
			IntPtr hCall, 
			string lpsUserUserInfo, 
			int dwSize
			);
		
		[DllImport("coredll", SetLastError=true)]
		extern static public int lineAnswer(
			IntPtr hCall,
			string lpsUserUserInfo,
			int dwSize
			);

        /// <bugs>
        /// <bug id="40">Cannot access the field MessageID from struct LineMessage</bug>
        /// </bugs>
        [DllImport("coredll", SetLastError = true)]
        public static extern int lineGenerateDigits(
            IntPtr hCall, 
            int dwDigitMode, 
            String lpszDigits, 
            int dwDuration
            );

		[DllImport("coredll", SetLastError=true)]
		extern static public int lineGetID(
			IntPtr hLine,
			int dwAddressID,
			IntPtr hCall,
			CallSelect dwSelect,
			/*LPVARSTRING*/
			byte[] lpDeviceID,
			string lpszDeviceClass
			);

		[DllImport("coredll", SetLastError=true)]
		extern static public int lineGetLineDevStatus(
			IntPtr hLine,
			/*LPLINEDEVSTATUS*/
			byte[] lpLineDevStatus
			);

		[DllImport("coredll", SetLastError=true)]
		extern static public int lineGetNewCalls(
			IntPtr hLine,
			int dwAddressID,
			CallSelect dwSelect,
			/*LPLINECALLLIST*/
			byte[] lpCallList
			);
	
		[DllImport("coredll", SetLastError=true)]
		extern static public int lineGetAddressID(
			IntPtr hLine,
			out int lpdwAddressID,
			AddressMode dwAddressMode,
			string lpsAddress,
			int dwSize 
			);

		[DllImport("coredll", SetLastError=true)]
		extern static public int lineGetAddressCaps(
			IntPtr hLineApp,
			int dwDeviceID,
			int dwAddressID,
			int dwAPIVersion,
			int dwExtVersion,
			/*LPLINEADDRESSCAPS*/
			byte[] lpAddressCaps 
			);

		[DllImport("coredll", SetLastError=true)]
		extern static public int lineGetAddressStatus(
			IntPtr hLine,
			int dwAddressID,
			/*LPLINEADDRESSSTATUS*/ 
			byte[] lpAddressStatus 
			);

		[DllImport("coredll", SetLastError=true)]
		extern static public int lineSetStatusMessages(
			IntPtr               hLine,
			DeviceState         dwLineStates,
			AddressState     dwAddressStates
			);


		[DllImport("cellcore")]
		extern static public int lineSetEquipmentState(
			IntPtr hLine,
			EquipmentState dwState
			);

		[DllImport("cellcore")]
		extern static public int lineGetEquipmentState(
			IntPtr hLine,
			out EquipmentState lpdwState,
			out RadioState lpdwRadioSupport
			);



		[DllImport("coredll", SetLastError=true)]
		extern static public int lineSetGPRSClass(
			IntPtr hLine,
			LineGprsClass dwClass
			);


		[DllImport("coredll", SetLastError=true)]
		extern static public int lineForward(
			IntPtr hLine,
			int bAllAddresses,
			int dwAddressID,
			ForwardList   lpForwardList,
			int               dwNumRingsNoAnswer,
			ref IntPtr        lphConsultCall,
			CallParameters    lpCallParams
		);

        public const string CELLTSP_PROVIDERINFO_STRING = "Cellular TAPI Service Provider";
        public const string CELLTSP_LINENAME_STRING = "Cellular Line";
        public const string CELLTSP_PHONENAME_STRING = "Cellular Phone";

        // Maximum string lengths
        public const int MAX_LENGTH_OPERATOR_LONG = 32;
        public const int MAX_LENGTH_OPERATOR_SHORT = 16;
        public const int MAX_LENGTH_OPERATOR_NUMERIC = 16;

        [DllImport("cellcore")]
        extern public static int lineGetCallBarringCaps(
                     IntPtr hLine,
                     out LineBarMode lpdwModes,
                     out int lpdwClasses
                     );
        /// <summary>
        /// New in v1.1
        /// </summary>
        /// <param name="hLine"></param>
        /// <param name="lpdwModes"></param>
        /// <param name="lpdwClasses"></param>
        /// <returns></returns>
        [DllImport("cellcore")]
        extern internal static int lineGetCallBarringCaps(
                     IntPtr hLine,
                     out LineBarMode lpdwModes,
                     out LineCapabilityClass lpdwClasses
                     );

        [DllImport("cellcore")]
        extern public static int lineGetCallBarringState(
            IntPtr hLine,
            LineBarMode dwMode,
            out int lpdwClasses,
            string lpszPassword
            );

        /// <summary>
        ///  New in v1.1
        /// </summary>
        /// <param name="hLine"></param>
        /// <param name="dwMode"></param>
        /// <param name="lpdwClasses"></param>
        /// <param name="lpszPassword"></param>
        /// <returns></returns>
        [DllImport("cellcore")]
        extern internal static int lineGetCallBarringState(
            IntPtr hLine,
            LineBarMode dwMode,
            out LineCapabilityClass lpdwClasses,
            string lpszPassword
            );

        [DllImport("cellcore")]
        extern public static int lineGetCallWaitingCaps(
            IntPtr hLine,
            out int lpdwClasses
            );

        /// <summary>
        ///  New in v1.1
        /// </summary>
        /// <param name="hLine"></param>
        /// <param name="lpdwClasses"></param>
        /// <returns></returns>
        [DllImport("cellcore")]
        extern internal static int lineGetCallWaitingCaps(
            IntPtr hLine,
            out LineCapabilityClass lpdwClasses
            );

        [DllImport("cellcore")]
        extern public static int lineGetCallWaitingState(
            IntPtr hLine,
            out int lpdwClasses
            );

        /// <summary>
        ///  New in v1.1
        /// </summary>
        /// <param name="hLine"></param>
        /// <param name="lpdwClasses"></param>
        /// <returns></returns>
        [DllImport("cellcore")]
        extern internal static int lineGetCallWaitingState(
            IntPtr hLine,
            out LineCapabilityClass lpdwClasses
            );

        [DllImport("cellcore")]
        extern public static int lineGetCurrentAddressID(
            IntPtr hLine,
            out int lpdwAddressID
            );

        [DllImport("cellcore")]
        extern public static int lineGetCurrentHSCSDStatus(
            IntPtr hLine,
            out int lpdwChannelsIn,
            out int lpdwChannelsOut,
            out int lpdwChannelCoding,
            out int lpdwAirInterfaceRate
            );

        [DllImport("cellcore")]
        extern public static int lineGetCurrentOperator(
            IntPtr hLine,
            /*LPLINEOPERATOR*/
            byte[] lpCurrentOperator
            );

        [DllImport("cellcore")]
        extern public static int lineGetCurrentSystemType(
            IntPtr hLine,
            out LineSystemType lpdwCurrentSystemType
            );

        [DllImport("cellcore")]
        extern public static int lineGetEquipmentState(
            IntPtr hLine,
            out int lpdwState,
            out RadioState lpdwRadioSupport
            );

        [DllImport("cellcore")]
        extern public static int lineGetGeneralInfo(
            IntPtr hLine,
            /*LPLINEGENERALINFO*/
            byte[] lpLineGeneralInfo
            );

        [DllImport("cellcore")]
        extern public static int lineGetGPRSClass(
            IntPtr hLine,
            out int lpdwClass
            );
        /// <summary>
        /// New in v1.1
        /// </summary>
        /// <param name="hLine"></param>
        /// <param name="lpdwClass"></param>
        /// <returns></returns>
        [DllImport("cellcore")]
        extern internal static int lineGetGPRSClass(
            IntPtr hLine,
            out LineGprsClass lpdwClass
            );

        [DllImport("cellcore")]
        extern public static int lineGetHSCSDCaps(
            IntPtr hLine,
            out int lpdwClass,
            out int lpdwChannelsIn,
            out int lpdwChannelsOut,
            out int lpdwChannelsSum,
            out int lpdwChannelCodings
            );

        [DllImport("cellcore")]
        extern public static int lineGetHSCSDState(
            IntPtr hLine,
            out int lpdwChannelsIn,
            out int lpdwMaxChannelsIn,
            out int lpdwChannelCodings,
            out int lpdwAirInterfaceRate
            );

        [DllImport("cellcore")]
        extern public static int lineGetMuteState(
            IntPtr hLine,
            out int lpdwState
            );

        /// <summary>
        /// New in v1.1
        /// </summary>
        /// <param name="hLine"></param>
        /// <param name="lpdwState"></param>
        /// <returns></returns>
        [DllImport("cellcore")]
        extern public static int lineGetMuteState(
            IntPtr hLine,
            out MuteState lpdwState
            );

        [DllImport("cellcore")]
        extern public static int lineGetNumberCalls(
            IntPtr hLine,
            out int lpdwNumActiveCalls,
            out int lpdwNumOnHoldCalls,
            out int lpdwNumOnHoldPendCalls
            );

        [DllImport("cellcore")]
        extern public static int lineGetOperatorStatus(
            IntPtr hLine,
            /*LPLINEOPERATORSTATUS*/
            byte[] lpOperatorStatus
            );

        [DllImport("cellcore")]
        extern public static int lineGetRadioPresence(
            IntPtr hLine,
            out RadioPresence lpdwRadioPresence
            );

        [DllImport("cellcore")]
        extern public static int lineGetRegisterStatus(
            IntPtr hLine,
            out RegistrationStatus lpdwRegisterStatus
            );

        [DllImport("cellcore")]
        extern public static int lineGetSendCallerIDState(
            IntPtr hLine,
            out int lpdwState
            );

        /// <summary>
        /// New in v1.1
        /// </summary>
        /// <param name="hLine"></param>
        /// <param name="lpdwState"></param>
        /// <returns></returns>
        [DllImport("cellcore")]
        extern internal static int lineGetSendCallerIDState(
            IntPtr hLine,
            out CallerIDState lpdwState
            );

        [DllImport("cellcore")]
        extern public static int lineGetUSSD(
            IntPtr hLine,
            int dwID,
            byte[] lpbUSSD,
            int dwUSSDSize,
            out int lpdwFlags
            );

        [DllImport("cellcore")]
        extern public static int lineRegister(
            IntPtr hLine,
            int dwRegisterMode,
            string lpszOperator,
            int dwOperatorFormat
            );

        //		[DllImport("cellcore")]
        //		extern public static int lineSendUSSD(
        //			IntPtr LinePtr,
        //			const BYTE* const lpbUSSD,
        //							int dwUSSDSize,
        //								int dwFlags
        //									);

        [DllImport("cellcore")]
        extern public static int lineSetCallBarringPassword(
            IntPtr hLine,
            int dwMode,
            string lpszOldPassword,
            string lpszNewPassword
            );

        /// <summary>
        ///  New in v1.1
        /// </summary>
        /// <param name="hLine"></param>
        /// <param name="dwMode"></param>
        /// <param name="lpszOldPassword"></param>
        /// <param name="lpszNewPassword"></param>
        /// <returns></returns>
        [DllImport("cellcore")]
        extern internal static int lineSetCallBarringPassword(
            IntPtr hLine,
            LineBarMode dwMode,
            string lpszOldPassword,
            string lpszNewPassword
            );

        [DllImport("cellcore")]
        extern public static int lineSetCallBarringState(
            IntPtr hLine,
            int dwMode,
            int dwClasses,
            string lpszPassword
            );

        /// <summary>
        ///  New in v1.1
        /// </summary>
        /// <param name="hLine"></param>
        /// <param name="dwMode"></param>
        /// <param name="dwClasses"></param>
        /// <param name="lpszPassword"></param>
        /// <returns></returns>
        [DllImport("cellcore")]
        extern internal static int lineSetCallBarringState(
            IntPtr hLine,
            LineBarMode dwMode,
            LineCapabilityClass dwClasses,
            string lpszPassword
            );

        [DllImport("cellcore")]
        extern public static int lineSetCallWaitingState(
            IntPtr hLine,
            int dwClasses,
            int dwState
            );

        /// <summary>
        /// New in version 1.1
        /// </summary>
        /// <param name="hLine"></param>
        /// <param name="dwClasses"></param>
        /// <param name="dwState"></param>
        /// <returns></returns>
        [DllImport("cellcore")]
        extern internal static int lineSetCallWaitingState(
            IntPtr hLine,
            LineCapabilityClass dwClasses,
            CallWaitingState dwState
            );

        [DllImport("cellcore")]
        extern public static int lineSetCurrentAddressID(
            IntPtr hLine,
            int dwAddressID
            );

        [DllImport("cellcore")]
        extern public static int lineSetEquipmentState(
            IntPtr hLine,
            int dwState
            );

        [DllImport("cellcore")]
        extern public static int lineSetGPRSClass(
            IntPtr hLine,
            int dwClass
            );

        [DllImport("cellcore")]
        extern public static int lineSetHSCSDState(
            IntPtr hLine,
            int dwChannelsIn,
            int dwMaxChannelsIn,
            int dwChannelCodings,
            int dwAirInterfaceRate
            );

        [DllImport("cellcore")]
        extern public static int lineSetMuteState(
            IntPtr hLine,
            int dwState
            );

        [DllImport("cellcore")]
        extern public static int lineSetSendCallerIDState(
            IntPtr hLine,
            int dwState
            );

        /// <summary>
        /// New in v1.1
        /// </summary>
        /// <param name="hLine"></param>
        /// <param name="dwState"></param>
        /// <returns></returns>
        [DllImport("cellcore")]
        extern internal static int lineSetSendCallerIDState(
            IntPtr hLine,
            CallerIDState dwState
            );

        [DllImport("cellcore")]
        extern public static int lineSetPreferredOperator(
            IntPtr hLine,
            /*LPLINEOPERATOR*/
            byte[] lpOperator
            );

        [DllImport("cellcore")]
        extern public static int lineUnregister(
            IntPtr hLine
            );

    }

}
