// -------------------------------------------------------
// This file is part of the OpenNETCF.Telephony Library
// 2007-2009 OpenNETCF Consulting, LLC
// www.opennetcf.com
//
// For license information visit http://tapi.codeplex.com
// -------------------------------------------------------

using System;
using System.Threading;
using System.Runtime.InteropServices;

namespace OpenNETCF.Telephony
{
	#region Async callback handling
			
	internal class TapiAsyncResult: IAsyncResult
	{
		protected int m_replyID;
        protected int m_response = -1;
		private AsyncCallback m_callback;
		private object m_state;
		private ManualResetEvent m_eventDone;

		internal virtual int ReplyID { set { m_replyID = value; } get{ return m_replyID; } }
		internal AsyncCallback Callback { get { return m_callback; } }
        internal int Response { set { m_response = value; } get { return m_response; } }

		internal TapiAsyncResult(int ReplyID, AsyncCallback callback, object State)
		{
			m_replyID = ReplyID;
			m_callback = callback;
			m_state = State;
		}

		#region IAsyncResult Members

		public object AsyncState
		{
			get
			{
				return m_state;
			}
		}

		public bool CompletedSynchronously
		{
			get
			{
				return m_replyID < 0;
			}
		}

		public WaitHandle AsyncWaitHandle
		{
			get
			{
				if ( m_eventDone == null )
					m_eventDone = new ManualResetEvent(false);
				return m_eventDone;
			}
		}

		public virtual bool IsCompleted
		{
			get
			{
				return false;
			}
		}

		#endregion

	}

	internal class MakeCallAsyncResult: TapiAsyncResult, IDisposable
	{
		internal IntPtr m_hCall;
		private GCHandle m_handle;
		internal MakeCallAsyncResult(int ReplyID, AsyncCallback callback, object State): base(ReplyID, callback, State)
		{
			m_handle = GCHandle.Alloc(this, GCHandleType.Pinned);
		}

		internal IntPtr hCall { get { return m_hCall; } }

		public override bool IsCompleted
		{
			get
			{
				return m_hCall != IntPtr.Zero || ReplyID <= 0;
			}
		}


		#region IDisposable Members

		public void Dispose()
		{
			m_handle.Free();
		}

		#endregion
	}

  

	#endregion

    #region CellCore AsyncHandlign

    internal class CellCoreAsyncResult : TapiAsyncResult
    {
        internal CellCoreAsyncResult( AsyncCallback callback, object State)
            : base(0, callback, State)
		{
		}

        internal override int ReplyID
        {
            get
            {
                return base.ReplyID;
            }
            set
            {
                if (value < 0)
                    throw new TelephonyException("Error in attempting to make asynchronous call.", value);
               
                base.ReplyID = value;
            }
        }

        public override bool IsCompleted
        {
            get
            {
                //tapi will set this to the response from the LineResponse message
                return m_response != -1;
            }
        }
    }

    internal class SetCallWaitingStateAsyncResult : CellCoreAsyncResult
    {
        bool m_On = false;
        internal SetCallWaitingStateAsyncResult(AsyncCallback callback, object State,bool on)
            : base(callback, State)
        {
            m_On = on;
        }

        public bool CallWaitingOn
        {
            get
            {
                return m_On;
            }
        }
    }

    internal class SetEquipmentStateAsyncResult : CellCoreAsyncResult
    {
        EquipmentState m_equipmentState;
        internal SetEquipmentStateAsyncResult( AsyncCallback callback, object State, EquipmentState equipmentState)
            : base(callback, State)
		{
            m_equipmentState = equipmentState;
		}

        public EquipmentState EquipmentState
        {
            get
            {
                return m_equipmentState;
            }
        }
    }

    internal class SetMuteStateAsyncResult : CellCoreAsyncResult
    {
        MuteState m_mutestate;
        internal SetMuteStateAsyncResult(AsyncCallback callback, object State, MuteState mutestate)
            : base(callback, State)
        {
            m_mutestate = mutestate;
        }

        public MuteState MuteState
        {
            get
            {
                return m_mutestate;
            }
        }
    }

    internal class SetCallerIDAsyncResult : CellCoreAsyncResult
    {
        CallerIDState m_state;
        internal SetCallerIDAsyncResult(AsyncCallback callback, object State, CallerIDState state)
            : base(callback, State)
        {
            m_state = state;
        }

        public CallerIDState CallerIDState
        {
            get
            {
                return m_state;
            }
        }
    }

    #endregion

}
